# -*- coding: utf-8 -*-
#    I, for one, welcome our new robot overloards
#    Copyright (C) 2008 David Eyk, David Thulson, Collin Sanford, David Howery
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""camera"""

from pyglet.gl import *
from rabbyt.primitives import Quad

class Camera(object):
    """Camera class to handle
    """
    def __init__(self, win_width, win_height):
        self.setPosition(0, 0)
        self.setWinWidthHeight(win_width, win_height)
        self._n_x = self._n_y = self._n_z = 0
        self._updateClipQuad(self._x, self._y)
            
    def getPosition(self):
        return self._x, self._y
    
    def setPosition(self, x, y):
        self._x = x
        self._y = y
        
    def translate(self, dx, dy):
        self._x += dx
        self._y += dy
        
    def setWinWidthHeight(self, win_width, win_height):
        self._win_width = win_width
        self._win_height = win_height
        
        self._half_win_w = self._win_width / 2.0
        self._half_win_h = self._win_height / 2.0
    
    def beforeDraw(self):
        # Store how to undo this translation
        x = self._x
        y = self._y
        self._n_x = x
        self._n_y = y
        glTranslatef(-x, -y, 0)
        self._updateClipQuad(x, y)
        return self._quad
    
    def afterDraw(self):
        glTranslatef(self._n_x, self._n_y, 0)
        self._n_x = self._n_y = 0
    
    def _updateClipQuad(self, x, y):
        self._quad = Quad((
            -self._half_win_w + x,
            self._half_win_h + y,
            self._half_win_w + x,
            -self._half_win_h + y
            ))
