# -*- coding: utf-8 -*-
#    I, for one, welcome our new robot overloards
#    Copyright (C) 2008 David Eyk, David Thulson, Collin Sanford, David Howery
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""entity"""

import operator

ai = None

from rsprite import RASprite
from enum import Enum
from vector2d import vec2Length

EntityCategories = Enum(
    'player',
    'AI',
    'weapon',
    'intersection',
    'building',
    'ground',
    'other')
AllCategories = frozenset(EntityCategories)
MovableCategories = frozenset(
    ( EntityCategories.player
    , EntityCategories.AI
    , EntityCategories.weapon
    , EntityCategories.other
    ))
CollidableCategories = frozenset(
    ( EntityCategories.player
    , EntityCategories.AI
    , EntityCategories.weapon
    , EntityCategories.building
    , EntityCategories.other
    ))
    
    
class Entity(RASprite):
    def __init__(self, texture, category=EntityCategories.other):
        self.health = 1
        self.alive = True
        
        self.velocity = [ 0.0, 0.0 ]
        
        self.mass = 0
        self.collision_elasticity = 0.8
        
        self._node = None
        self._world = None
        self._category = category
        self._is_movable = category in MovableCategories
        super(Entity, self).__init__(texture)
        self._prev_x = self.x
        self._prev_y = self.y

        global ai
        if ai is None:
            import ai
        ai.Root.registerObject(self, category)
    
    def updateNode(self):
        # If this isn't part of a scene, don't do anything
        if self._is_movable and (self._node is not None):
            # Only update it if the position has changed
            x, y = self.xy
            if (self._prev_x != x) or (self._prev_y != y):
                self._node = self._node.updateEntity(self)
                assert (self._node is None) or self._node.isLeaf(), "Internal error updating entity"
                self._prev_x = x
                self._prev_y = y

    def setNode(self, node):
        """Do not call this function except from the quadtree!
        """
        assert (node is None) or node.isLeaf(), "Adding non-leaf node to entity"
        self._node = node
    
    def setWorld(self, world):
        """Do not call this function except from the quadtree!
        """
        self._world = world
    
    def kill(self):
        self.alive = False
    
    def damage(self, amt):
        pass
    
    def collideWith(self, obj):
        pass
    
    def getPrevXY(self):
        return self._prev_x, self._prev_y
    
    def _get_speed(self):
        return vec2Length(self.velocity)
    
    category = property(operator.attrgetter('_category'))
    world = property(operator.attrgetter('_world'))
    speed = property(_get_speed)
