# -*- coding: utf-8 -*-
#    I, for one, welcome our new robot overloards
#    Copyright (C) 2008 David Eyk, David Thulson, Collin Sanford, David Howery
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""menus -- menus for projectx

$Author: csanford $
$Rev: 1038 $
$Date: 2008-04-05 18:36:37 -0500 (Sat, 05 Apr 2008) $
"""

__author__ = "$Author: csanford $"
__revision__ = "$Rev: 1038 $"
__date__ = "$Date: 2008-04-05 18:36:37 -0500 (Sat, 05 Apr 2008) $"

from time import clock
from math import sqrt

from random import randint, choice

import operator
_getZ = operator.attrgetter('z')

import rabbyt
from rabbyt import add_time, scheduler
pump = scheduler.pump

from pyglet import font
from pyglet import image

from cocos.director import director

from cocos import scene
from cocos import layer
from cocos import transitions
from cocos import actions

import resources
import actors
import rsprite
import menus
import terrain
from sound import sound

import world
import camera
from entity import EntityCategories
from vector2d import *

from pyglet.window import key as keys


def simpleCollisionResponse(a, b):
    a_mass = a.mass
    b_mass = b.mass
    if (a_mass == 0) and (b_mass == 0):
        return
    
    dx = a.x - b.x
    dy = a.y - b.y
    dist = sqrt(dx*dx + dy*dy)
    needed_dist = (a.bounding_radius + b.bounding_radius) - dist
    
    # Not sure why we get a callback if this happens, but oh well...
    if needed_dist <= 0:
        return
    
    v = (needed_dist * (dx / dist), needed_dist * (dy / dist))
    
    to_move = None
    mult = 1
    if a_mass == 0:
        to_move = b
        mult = -1
    elif b_mass == 0:
        to_move = a
        mult = 1
    else:
        if a_mass < b_mass:
            to_move = a
            mult = 1
        else:
            to_move = b
            mult = -1
    
    to_move.x += mult * v[0]
    to_move.y += mult * v[1]
    
def betterCollisionResponse(a, b):
    a_mass = a.mass
    b_mass = b.mass
    if (a_mass == 0) and (b_mass == 0):
        return
    
    delta = [ a.x - b.x, a.y - b.y ]
    dist = vec2Length(delta)
    needed_dist = (a.bounding_radius + b.bounding_radius) - dist
    
    if (delta[0] == 0) and (delta[1] == 0):
        delta[0] = 1.0
    
    # Not sure why we get a callback if this happens, but oh well...
    if needed_dist < 0:
        return
    
    a_prev_x, a_prev_y = a.getPrevXY()
    b_prev_x, b_prev_y = b.getPrevXY()
    prev_dist = vec2Length([ a_prev_x - b_prev_x, a_prev_y - b_prev_y ])
    move_to_prev = prev_dist > dist
    
    vec2Normalize(delta)
    rel_v = vec2Sub(a.velocity, b.velocity)
    if rel_v[0] == 0 and rel_v[1] == 0:
        rel_v[0] = 0.1
    
    elasticity = min(a.collision_elasticity, b.collision_elasticity)
    
    a_mass_inv = 0
    b_mass_inv = 0
    if a_mass > 0:
        a_mass_inv = 1.0/a_mass
    if b_mass > 0:
        b_mass_inv = 1.0/b_mass
    
    j = (-(1+elasticity) * vec2Dot(rel_v, delta)) / (vec2Dot(delta, delta) * (a_mass_inv + b_mass_inv))
    
    if a_mass > 0:
        if move_to_prev:
            a.xy = (a_prev_x, a_prev_y)
        a.velocity[0] += (j * delta[0]) / a_mass
        a.velocity[1] += (j * delta[1]) / a_mass
    if b_mass > 0:
        if move_to_prev:
            b.xy = (b_prev_x, b_prev_y)
        b.velocity[0] -= (j * delta[0]) / b_mass
        b.velocity[1] -= (j * delta[1]) / b_mass

def noOpCollideHandler(a, b):
    pass
    
def defaultCollideHandler(a, b):
    if a.alive and b.alive:
        a.collideWith(b)
        b.collideWith(a)
        # Only moving when both are still alive looks a little better for
        #   running over the dude, not sure if it will make sense with better
        #   collision response or other collision types
        if a.alive and b.alive:
            betterCollisionResponse(a, b)


class WorldScene(scene.Scene):
    def __init__(self):
        super(WorldScene, self).__init__()
        self.wl = WorldLayer()
        self.add(1, self.wl, 'world')
        self.add(2, InfoLayer(self.wl), 'info')

    def step(self, dt):
        add_time(dt)
        pump()
        scene.Scene.step(self, dt)


class WorldLayer(layer.Layer):
    player_health = property(lambda self: self.player.health * 100)
    dudes_left = property(lambda self: len([i for i in self.dudes if i.alive]))
    enemy_health = property(lambda self: self.enemy.health * 100)
    
    def __init__(self):
        # This doesn't work because it messes up cocos
        #rabbyt.set_viewport((800, 600))
        #rabbyt.set_default_attribs()
        
        super(WorldLayer, self).__init__()
        self.world = world.World(resources.getFilePath('Terrain.rter'))
        # Set the default collision handler to our default
        self.world.setDefaultCollisionHandler(defaultCollideHandler)
        # As an example of how you can override the default
        # I'm not sure we really want weapons to collide, so we can try it by
        #   doing this...
        self.world.registerCollisionHandler(
            EntityCategories.weapon, EntityCategories.weapon, noOpCollideHandler)
        # Not sure if this will happen, but it might since the building tiles
        #   all border each other
        self.world.registerCollisionHandler(
            EntityCategories.building, EntityCategories.building, noOpCollideHandler)
        # Monkey-patching the world to allow for the changing of points 
        # reasonably easily for entities
        self.world.change_points = self.change_points
        self.camera = camera.Camera(800, 600) # HACK: Need to grab res from director

        self.player = actors.Robot(enemy=False)
        self.player.xy = (128, 128)
        self.player.z = 1
        self.addEntity(self.player)
        assert self.player.mass != 0, "Player mass is 0"

        self.enemy = actors.Robot(enemy=True)
        self.enemy.xy = (1200, 800)
        self.enemy.z = 1
        self.addEntity(self.enemy)
        assert self.enemy.mass != 0, "Player mass is 0"        
        
        self.dudes = []
        for x in xrange(20):
            dude = actors.Dude()
            self.dudes.append(dude)
            dude.xy = (randint(500, 700), randint(400, 700))
            dude.z = .5
            self.addEntity(dude)
        
        self.paused = False
        
        self.points = 0
        self.message = "Don't run, we are your friends."
        self._end_game = False
        self._end_game_time = 0

    def addEntity(self, e):
        self.world.add(e)
        
    def change_points(self, dpoints, message=None):
        self.points += dpoints
        if message:
            self.message = message
            
    def check_end_game(self):
        if (self.player.health <= 0) \
                or (self.dudes_left <= 0):
            self.message = "YOU LOSE LOSER!"
            self._end_game = True            

    def step(self, dt):
        if not self.paused:
            if self._end_game:
                self._end_game_time += dt
                if self._end_game_time > 5:
                    director.pop()

            # Move entities...
            # Should this only update movable entities?  Should it only update
            #  entities within a certain range?
            [ x.step(dt) for x in self.world.queryAllEntities() ]
            
            # This doesn't work
            self.camera.setPosition(self.player.x - 400, self.player.y - 300)

            # Collide entities...
            # The ordering here will have to change if we pass this a bounding
            #   box
            self.world.handleCollisions()

            # Update quadtree...
            self.world.update()
            
            self.check_end_game()
            
            #update the sound singleton
            sound.update()
            
            # Set camera translation and get clipping info
            clip_quad = self.camera.beforeDraw()
            try:
                clip_quad.width += 64
                clip_quad.height += 64
                # This is weird but we have to account for the origin difference
                clip_quad.left += 400
                clip_quad.bottom += 300

                # Render!
                to_render = self.world.queryEntitiesWithin(clip_quad)
                #to_render = self.world.queryAllEntities()
                to_render.sort(key=_getZ)
                rabbyt.render_unsorted(to_render)
            finally:
                # Undo camera movement
                self.camera.afterDraw()

    def on_key_press (self, symbol, modifiers):
        """This function is called when a key is pressed.
        
        'key' is a constant indicating which key was pressed.
        'modifiers' is a bitwise or of several constants indicating which
        modifiers are active at the time of the press (ctrl, shift, capslock, etc.)
        
        """
        if symbol == keys.ESCAPE:
            self.on_quit()
            return True

        return self.player.on_key_press(symbol, modifiers)
        
    def on_key_release (self, symbol, modifiers):
        """This function is called when a key is released.
        
        'key' is a constant indicating which key was pressed.
        
        'modifiers' is a bitwise or of several constants indicating
           which modifiers are active at the time of the press (ctrl,
           shift, capslock, etc.)

        Constants are the ones from pyglet.window.key
        """
        return self.player.on_key_release(symbol, modifiers)

    def on_quit(self):
        sound.stop()
        director.replace(transitions.GrowTransition(director.scene, 
                                                    director.scene_stack.pop(),
                                                    0.5))
        
class InfoLayer(layer.Layer):
    def __init__(self, world_layer):
        super(InfoLayer, self).__init__()
        self.world_layer = world_layer
        
        self.font = resources.loadFont('Gentium', 20)
        self.font_color = (1,1,1,1)
        self.message = font.Text(self.font, color=self.font_color)
        self.player_health = font.Text(self.font, color=self.font_color)
        self.enemy_health = font.Text(self.font, color=self.font_color)
        self.dudes_left = font.Text(self.font, color=self.font_color)
        self.points = font.Text(self.font, color=self.font_color)
        self._message_time = 1
        self._since_last_change = 0
        
    def _set_alignments(self):
        buffer = 5
        width, height = director.window.width, director.window.height
        ph = self.player_health
        ph.x = width - ph.width
        ph.y = height - ph.height - buffer

        dl = self.dudes_left
        dl.x = width - dl.width - buffer
        dl.y =  ph.y - ph.height - buffer

        eh = self.enemy_health
        eh.x = width - eh.width - buffer
        eh.y = dl.y - dl.height - buffer
        
        p = self.points
        p.x = width - p.width - buffer
        p.y = eh.y - eh.height - buffer
        
        m = self.message
        m.x = width - m.width - buffer
        m.y = p.y - p.height - buffer
        
    def step(self, dt):
        wl = self.world_layer

        self.message.text = wl.message

        self.player_health.text = "Health: %d" % wl.player_health
        self.dudes_left.text = "Dudes Left: %d" % wl.dudes_left
        self.enemy_health.text = "Enemy: %d" % wl.enemy_health
        self.points.text = "Total Points: %d" % wl.points
        
        self._set_alignments()
        
        self.message.draw()
        self.player_health.draw()
        self.dudes_left.draw()
        self.enemy_health.draw()
        self.points.draw()

class CarDemo(scene.Scene):
    def __init__(self):
        super(CarDemo, self).__init__()
        self.add(5, ActorsLayer(), 'actors')
        self.add(1, RoadLayer(), 'road')

    def step(self, dt):
        add_time(dt)
        pump()
        super(CarDemo, self).step(dt)


class RoadLayer(layer.Layer):
    def __init__(self):
        super(RoadLayer, self).__init__()
        self.road = rsprite.RASprite(resources.getFilePath('road.png'))
        self.road.top = 600
        self.road.left = 0

    def step(self, dt):
        self.road.render()


class ActorsLayer(layer.Layer):
    p_xy = property(lambda self: "x, y: %s" % (self.player.xy,))
    p_tl = property(lambda self: "t, l: (%s, %s)" % (self.player.top, 
                                                     self.player.left))
    p_br = property(lambda self: "b, r: (%s, %s)" % (self.player.bottom,
                                                     self.player.right))
    p_wh = property(lambda self: "w, h: (%s, %s)" % (
            self.player.right-self.player.left,
            self.player.top-self.player.bottom))

    p_rot = property(lambda self: "rot: %s" % (self.player.rot,))

    def __init__(self):
        super(ActorsLayer, self).__init__()

        self.display_font = resources.loadFont('Gentium', 14)

        self.keys_pressed = set()

        self.sprites = []

        self.player = actors.Car()
        self.player.x = 30
        self.player.y = 560
        self.player.face(300, 400)

        self.pedestrian = actors.Dude()
        self.pedestrian.xy = 400, 300

        self.sprites.append(self.player)
        self.sprites.append(self.pedestrian)

        self.display_xy = font.Text(self.display_font, self.p_xy,
                                    x=15, y=500)
        self.display_tl = font.Text(self.display_font, self.p_tl,
                                    x=15, y=480)
        self.display_br = font.Text(self.display_font, self.p_br,
                                    x=15, y=460)
        self.display_wh = font.Text(self.display_font, self.p_wh,
                                    x=15, y=440)
        self.display_rot = font.Text(self.display_font, self.p_rot,
                                    x=15, y=420)

    def update_text(self):
        self.display_xy.text = self.p_xy
        self.display_tl.text = self.p_tl
        self.display_br.text = self.p_br
        self.display_wh.text = self.p_wh
        self.display_rot.text = self.p_rot

        self.display_xy.draw()
        self.display_tl.draw()
        self.display_br.draw()
        self.display_wh.draw()
        self.display_rot.draw()
        

    def step(self, dt):
        sprites = self.sprites
        [s.step(dt) for s in sprites]
        rabbyt.render_unsorted(sprites)
        self.update_text()

        

    def on_key_press (self, symbol, modifiers):
        """This function is called when a key is pressed.
        
        'key' is a constant indicating which key was pressed.
        'modifiers' is a bitwise or of several constants indicating which
        modifiers are active at the time of the press (ctrl, shift, capslock, etc.)
        
        """
        if symbol == keys.ESCAPE:
            self.on_quit()
            return True

        return self.player.on_key_press(symbol, modifiers)
        
    def on_key_release (self, symbol, modifiers):
        """This function is called when a key is released.
        
        'key' is a constant indicating which key was pressed.
        
        'modifiers' is a bitwise or of several constants indicating
           which modifiers are active at the time of the press (ctrl,
           shift, capslock, etc.)

        Constants are the ones from pyglet.window.key
        """
        return self.player.on_key_release(symbol, modifiers)

    def on_quit(self):
        director.replace(transitions.GrowTransition(director.scene, 
                                                    director.scene_stack.pop(),
                                                    0.5))
