# -*- coding: utf-8 -*-
#    I, for one, welcome our new robot overloards
#    Copyright (C) 2008 David Eyk, David Thulson, Collin Sanford, David Howery
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""menus -- menus for projectx

$Author: eykd $
$Rev: 964 $
$Date: 2008-04-04 14:35:38 -0500 (Fri, 04 Apr 2008) $
"""

__author__ = "$Author: eykd $"
__revision__ = "$Rev: 964 $"
__date__ = "$Date: 2008-04-04 14:35:38 -0500 (Fri, 04 Apr 2008) $"

import sys

from pyglet import font
from pyglet.window import key

from cocos.director import director

from cocos import scene
from cocos import layer
from cocos import menu
from cocos import transitions

import resources
import game
import rsprite
import events

def doNothing(*args, **kwargs):
    pass

class EntryMenuItem(menu.MenuItem):
    value = property(lambda self: u''.join(self._value))

    def __init__(self, label, value, activate_func):
        super(EntryMenuItem, self).__init__(unicode(label), activate_func)
        self._value = unicode(value).split()

    def on_mouse_release( self, x, y, buttons, modifiers ):
        self.selected = True
        return True

    def on_key_press(self, symbol, modifiers):
        result = False
        if symbol == key.BACKSPACE:
            try:
                self._value.pop()
            except IndexError:
                pass
            result = True
        elif symbol in events.CHARACTERS:
            char = events.KEY_MAP[symbol]
            if key.MOD_SHIFT & modifiers:
                char = char.upper()
            self._value.append(char)
            result = True

        if result:
            value = self.value
            new_text = u"%s %s" % (self.label, 
                                   value)
            self.text.text = new_text
            self.text_selected.text = new_text
            self.activate_func(value)
        return result
                    

class MainScene(scene.Scene):
    def __init__(self):
        n = 0.75
        color_screen = (n, n, n, n)
        super(MainScene, self).__init__(MainMenuBackground(), 
                                        #layer.ColorLayer(*color_screen),
                                        MainMenu())


class MainMenuBackground(layer.Layer):
    def __init__(self):
        super(MainMenuBackground, self).__init__()
        img = resources.loadImage("RobotsAreYourFriends.png")
        self.bg = rsprite.RASprite(img.texture)
        self.bg.width = 800
        self.bg.top = 600
        self.bg.left = 0
        self.bg.alpha = .5

    def step(self, dt):
        self.bg.render()


class MainMenu(menu.Menu):
    def __init__(self):
        super(MainMenu, self).__init__("I For One WELCOME Our New Robot Overlords.")
        self.menu_halign = menu.LEFT
        
        self.font_title = 'Gentium'
        self.font_title_size = 28
        self.font_title_color = (1,1,1,1)

        self.font_items = 'Gentium'
        self.font_items_size = 24
        self.font_items_selected_size = 26

        self.font_items_color = (0.7, 0.7, 0.7, 1)

        self.add_item(EntryMenuItem("Enter your name: ", "", doNothing))

        # It's just not that impressive any more.
        #self.add_item(menu.MenuItem("Car Demo", self._goDemo))

        self.add_item(menu.MenuItem("New Game", self._goGame))

        self.build_items()

    def _goDemo(self):
        t = transitions.FadeTransition(self, game.CarDemo(), 1)
        director.push(t)

    def _goGame(self):
        t = transitions.FadeTransition(self, game.WorldScene(), 1)
        director.push(t)

    def on_quit(self):
        sys.exit()
