# -*- coding: utf-8 -*-
#    I, for one, welcome our new robot overloards
#    Copyright (C) 2008 David Eyk, David Thulson, Collin Sanford, David Howery
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""rsprites -- Rabbyt sprites for cocos.

$Author: eykd $
$Rev: 1026 $
$Date: 2008-04-05 17:50:49 -0500 (Sat, 05 Apr 2008) $
"""

__author__ = "$Author: eykd $"
__revision__ = "$Rev: 1026 $"
__date__ = "$Date: 2008-04-05 17:50:49 -0500 (Sat, 05 Apr 2008) $"

from math import cos, sin, acos, asin, tan, atan2
from math import radians, degrees
from math import pi
pi_2 = pi*2
pi_1_2 = pi/2.0

import operator

import copy

import rabbyt

from rabbyt import sprites

from cocos.euclid import Point3

from cocos.actions import Animation

class RASprite(sprites.Sprite):
    """Behold, the Rabbyt/Action Sprite!

    This is a Rabbyt Sprite that plays nicely with the Cocos scene
    manager API.
    """
    angle = property(operator.attrgetter('rot'),
                     lambda self, a: setattr(self, 'rot', a))

    frame = property(operator.attrgetter('texture'),
                     lambda self, v: setattr(self, 'texture', v.texture))

    draw = sprites.Sprite.render

    color = sprites.Sprite.rgba

    def __init__(self, texture):#, shape=None, tex_shape=(0,1,1,0)):
        super(RASprite, self).__init__(texture)#, shape, tex_shape)

        self.actions = []
        self.to_remove = []
        self.translate = Point3(0,0,0)
        self.show = True
        self.animations = {}
        self.z = 0.0

        default_anim = Animation("default", 0, texture)
        self.add_animation(default_anim)

    def getVectorComponents(self):
        x = cos(radians(self.rot))
        y = sin(radians(self.rot))
        return x, y

    def getRotationDelta(self, t_x, t_y):
        my_x, my_y = self.xy
        dx = my_x - t_x
        dy = my_y - t_y
        facing = self.getFacing(t_x, t_y)
        rot_delta = radians(self.rot) - atan2(dy, dx) + pi
        while rot_delta < -pi:
            rot_delta += pi_2
        while rot_delta > pi:
            rot_delta -= pi_2

        return rot_delta

    def getFacing(self, x, y):
        return degrees(atan2(y, x))-90.0

    def face(self, x, y):
        self.rot = self.getFacing(x, y)

    def do(self, action):
        '''Executes an *action*.
        When the action finished, it will be removed from the sprite's queue.

        :Parameters:
            `action` : an `Action` instance
                Action that will be executed.
        :rtype: `Action` instance
        :return: A clone of *action*
        '''
        try:
            # HACK:
            # deepcopy is needed to run the same sequence of actions
            # in different sprites at the same time
            a = copy.deepcopy( action )
        except Exception, e:
            # but deepcopy fails when copying the CallFunc calling an instance
            # method.
            print "WARNING: Running this action with various sprites at the time has an unpredictable behaviour"
            print "CallFunc / CallFuncS actions can't be deep-copied when they are calling instance methods."
            print "Workaround: Make it call a free function instead."
            print action
            a = copy.copy( action )

        a.target = self
        a._start()
        self.actions.append( a )
        return a

    def done(self, action ):
        self.to_remove.append( action )

    def step(self, dt):
        """This functions is called *n* times per second, where
        *n* are the FPS.

        :Parameters:
            `dt` : delta_time
                The time that ellapsed since that last time this functions was called.
        """
        for action in self.actions:
            action._step(dt)
            if action.done():
                self.done( action )
                
        remove = self.actions.remove
        [remove(x) for x in self.to_remove]
        self.to_remove = []

    def place(self, coords):
        self.center = coords[0:2]

    xyz = rabbyt.swizzle('x', 'y', 'z')

    translate = property(lambda self: Point3(*self.xyz),
                         place)

    box = rabbyt.swizzle('left', 'right', 'top', 'bottom')

    def get_box(self):
        return self.box

    def add_animation(self, animation):
        self.animations[animation.name] = animation

    

