# -*- coding: utf-8 -*-
#    I, for one, welcome our new robot overloards
#    Copyright (C) 2008 David Eyk, David Thulson, Collin Sanford, David Howery
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License along
#    with this program; if not, write to the Free Software Foundation, Inc.,
#    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

""" A map creator """

import sys
import os
import copy

import pyglet
from pyglet import gl
from pyglet.window import mouse, key
from pyglet import font
from pyglet import event
import cocos
from cocos.director import director
from cocos import menu
import rabbyt

from projectx import terrain
from projectx import resources
from projectx.camera import Camera
from projectx.rsprite import RASprite
from projectx.entity import EntityCategories

sWidth,sHeight = 1024,800

def GetClosestValue(integer, list_of_values):
    dif = 0
    min_dif = 10000
    return_value = 0
    for i in list_of_values:
        dif = abs(integer - i)
        if dif == 0:
            return i
        if dif < min_dif:
            min_dif, return_value = dif, i
    return return_value

class curry:
    def __init__(self, fun, *args, **kwargs):
        self.fun = fun
        self.pending = args[:]
        self.kwargs = kwargs.copy()

    def __call__(self, *args, **kwargs):
        if kwargs and self.kwargs:
            kw = self.kwargs.copy()
            kw.update(kwargs)
        else:
            kw = kwargs or self.kwargs

        return self.fun(*(self.pending + args), **kw)

class Button(RASprite):
    def __init__(self, text, event):
        super(Button, self).__init__(resources.getFilePath('Button.png'))
        self.__font = resources.loadFont('Gentium', 14)
        self.__button_text = font.Text(self.__font, text)
        self.scale = ((self.__button_text.width + 10) / float(self.texture.width), 
                      self.__button_text.height / float(self.texture.height))
        self.__event = event
        
    def set_text(self, text):
        self.__button_text.text = text
        
    text = property(lambda self: self.__button_text.text, set_text)
    width = property(lambda self: self.texture.width * self.scale[0])
        
    def step(self, dt):
        self.draw()
        self.__button_text.x = self.x - (0.5 * self.__button_text.width)
        self.__button_text.y = self.y - 5
        self.__button_text.draw()
        
    def on_mouse_press(self, x, y, button, modifiers):
        m_x, m_y = director.get_virtual_coordinates(x,y)
        c = rabbyt.collisions.collide_single((m_x, m_y), (self,))
        if c:
            self.scale = (-abs(self.scale_x), -abs(self.scale_y))
            return True
        return False
    
    def on_mouse_release(self, x, y, button, modifiers):
        m_x, m_y = director.get_virtual_coordinates(x,y)
        c = rabbyt.collisions.collide_single((m_x, m_y), (self,))
        if c:
            self.scale = (abs(self.scale_x), abs(self.scale_y))
            self.__event()
            return True
        return False
            
    def on_mouse_motion(self, x, y, dx, dy):
        m_x, m_y = director.get_virtual_coordinates(x,y)
        c = rabbyt.collisions.collide_single((m_x, m_y), (self,))
        if c:
            self.rgb = (0.9, 0.9, 0.9)
        else:
            self.rgb = (1, 1, 1)
        return False

class EditorMenu(menu.Menu):
    def __init__(self, terrain_layer):
        super(EditorMenu, self).__init__('')
        self.font_title_size = 15
        self.font_items_size = 15
        self.font_items_color = (0,255, 0, 255)
        self.font_items_selected_size = 15
        self.font_items_selected_color = (255,0,0,255)
        self.menu_halign = menu.LEFT
        self.menu_valign = menu.TOP
        self.terrain = terrain_layer

class NewTile(EditorMenu):
    def __init__(self, terrain_layer):
        super(NewTile, self).__init__(terrain_layer)
        
        path = resources.getFilePath('')
        l = os.listdir(path)
        l.sort()
        for i in l:
            if os.path.isfile(os.path.join(path, i)) and os.path.splitext(i)[1] == '.png':
                self.add_item(menu.MenuItem(i, curry(self.__useFile, i)))
        self.add_item(menu.MenuItem('Back', self.__onBack))
        
        self.build_items()
    
    def __useFile(self, filename):
        self.terrain.add_texture_as_tile(filename)
        self.switch_to(0)
        
    def __onBack(self):
        self.switch_to(0)
        
    def on_quit(self):
        self.__onBack()

class Main(EditorMenu):
    def __init__(self, terrain_layer):
        super(Main, self).__init__(terrain_layer)
        
        self.add_item(menu.MenuItem("New Tile", self.__onNewTile))
        self.add_item(menu.MenuItem("Save Map", self.__onSave))
        self.add_item(menu.MenuItem("Quit", self.on_quit))
        
        self.build_items()
        
    def createTileCallback(self, TextureName):
        print TextureName
        
    def __onNewTile(self):
        self.switch_to(1)
    
    def __onSave(self):
        self.terrain.Save()
    
    def on_quit(self):
        sys.exit()        

class Grid(cocos.layer.Layer):
    """A Grid of lines to snap each tile to."""
    def __init__(self, terrain, size=64):
        super(Grid, self).__init__()
        self.__obj = []
        self.__gridSize = size
        self.__textureName = resources.getFilePath('Grid.png')
        
        self.__setupGrid(sWidth, sHeight, self.__gridSize)
        terrain.set_collision_objs(self.__obj)
        
    def step(self, dt):
        for o in self.__obj:
            o.render()
            
    def __setupGrid(self, width, height, size):
        self.__obj = []
        rabbyt.set_viewport((width, height))
        for x in range(-32, int(width) + 32, size):
            for y in range(-32, int(height) + 32, size):
                self.__obj.append(rabbyt.Sprite(
                        texture=self.__textureName,
                        xy=(x,y),
                        scale=1))
    
    def on_resize(self, width, height):
        x, y = director.get_virtual_coordinates(width, height)
        self.__setupGrid(x, y, 64)
        
class Terrain(cocos.layer.Layer):
    """A Layer to display our terrain."""
    def __init__(self):
        super(Terrain, self).__init__()
        self.terrain = terrain.Terrain(self)
        self.__obj = []
        self.__co = []
        self.__tracked_tile = None
        self.__selected = []
        self.__new = None
        self.terrain.LoadTerrain(open(resources.getFilePath('Terrain.rter')).read(), False)
        self.__camera = Camera(sWidth, sHeight)
        self.__camera.setPosition(-32, -32)
        self.display_font = resources.loadFont('Gentium', 14)
        
        self.__origin = font.Text(self.display_font, '(0,0)', x=-32, y=-32, color=(1,0,0,1))
        green = (0, 1, 0, 1)
        
        self.__scale_x_btn = Button("FlipH", self.__scale_x)
        self.__scale_x_btn.xy = (575, 520)
        self.__scale_y_btn = Button("FlipV", self.__scale_y)
        self.__scale_y_btn.xy = (650, 520)
        self.display_texture_name = font.Text(self.display_font, '', x=550, y=500, color=green)
        self.display_damage_name = font.Text(self.display_font, '', x=550, y=480, color=green)
        self.display_blocked = font.Text(self.display_font, 'Blocked: ', x=550, y=460, color=green)
        self.__blocked_btn = Button('False', self.__toggle_blocked)
        self.__blocked_btn.xy = (600 + self.display_blocked.width, 460)
        self.display_strength = font.Text(self.display_font, '', x=550, y=440, color=green)
        self.__rotate_btn = Button('Rotate', self.__rotate)
        self.__rotate_btn.xy = (550 + (self.__rotate_btn.width / 2), 420)
        self.__delete_btn = Button('Delete', self.__delete_tile)
        self.__delete_btn.xy = (550 + (self.__delete_btn.width / 2), 400)
        self.__copy_btn = Button('Copy', self.__copy_tile)
        self.__copy_btn.xy = (550 + (self.__copy_btn.width / 2), 380)
        
    def __scale_x(self):
        for i in self.__selected:
            i.scale_x = -i.scale_x
        
    def __scale_y(self):
        for i in self.__selected:
            i.scale_y = -i.scale_y
        
    def __toggle_blocked(self):
        for i in self.__selected:
            i.blocked = not i.blocked
            i.strength = 100 if i.blocked else 0
        
    def __rotate(self):
        for i in self.__selected:
            i.rot += 90
            if i.rot >= 360:
                i.rot = 0
        
    def __delete_tile(self):
        
        for i in self.__selected:
            try:
                self.__obj.remove(i)
            except ValueError:
                pass
        self.__selected = []
        
    def __copy_tile(self):
        for i in self.__selected:
            n = terrain.Tile(i._texture_name,
                             i._damaged_name,
                             i.blocked,
                             i.strength,
                             i.rot)
            n.scale = i.scale
            self.__new = n
            self.__obj.append(n)
        
    def add(self, obj):
        self.__obj.append(obj)
        
    def set_collision_objs(self, objs):
        self.__co = objs
        
    def add_texture_as_tile(self, texture_file):
        t = terrain.Tile(texture_file,
                         texture_file,
                         location=(0,0))
        self.__new = t
        self.__obj.append(t)
        
    def __to_world(self, x, y):
        wx, wy = self.__camera.getPosition()
        return x + wx, y + wy
    
    def __from_world(self, x, y):
        wx, wy = self.__camera.getPosition()
        return x - wx, y - wy
    
    def __draw_info(self, dt):
        self.__rotate_btn.step(dt)
        self.__delete_btn.step(dt)
        self.__copy_btn.step(dt)
        self.__scale_x_btn.step(dt)
        self.__scale_y_btn.step(dt)
        
        self.__blocked_btn.text = str(self.__selected[-1].blocked)
        self.__blocked_btn.step(dt)
        
        self.display_texture_name.text = "Tex: %s" % self.__selected[-1]._texture_name
        self.display_damage_name.text = "Dam: %s" % self.__selected[-1]._damaged_name
        
        self.display_strength.text = "Strength: %d" % self.__selected[-1].strength
        
        self.display_texture_name.draw()
        self.display_damage_name.draw()
        self.display_blocked.draw()
        self.display_strength.draw()
        
    def step(self, dt):
        self.__camera.beforeDraw()
        try:
            for o in self.__obj:
                o.render()
            self.__origin.draw()
        finally:
            self.__camera.afterDraw()        

        if self.__selected:
            self.__draw_info(dt) 
            
    def Save(self):
        self.terrain.SaveTerrain('Terrain.rter', self.__obj)
        
    def __buttonActions(self, action_name, *args):
        if self.__selected:
            return Button.__dict__[action_name](self.__rotate_btn, *args) \
                 or Button.__dict__[action_name](self.__delete_btn, *args)\
                 or Button.__dict__[action_name](self.__copy_btn, *args) \
                 or Button.__dict__[action_name](self.__blocked_btn, *args) \
                 or Button.__dict__[action_name](self.__scale_x_btn, *args) \
                 or Button.__dict__[action_name](self.__scale_y_btn, *args)
        return False
            
    def on_mouse_press(self, x, y, button, modifiers):
        m_x, m_y = director.get_virtual_coordinates(x,y)
        w_x, w_y = self.__to_world(m_x, m_y)
        if button == mouse.LEFT:
            if self.__buttonActions('on_mouse_press', x, y, button, modifiers):
                return True
            if self.__new:
                c = rabbyt.collisions.collide_single((m_x,m_y), self.__co)
                for i in c:
                    self.__new.xy = self.__to_world(i.x, i.y)
                self.__new = None
                return True
            else:
                c = rabbyt.collisions.collide_single((w_x, w_y), self.__obj)
                if c:
                    c[-1].alpha = 0.5
                    self.__tracked_tile = c[-1]
                else:
                    self.__tracked_tile = None
                return True
        elif button == mouse.MIDDLE:
            self.__starting_pos = (w_x, w_y)
        return False
    
    def on_mouse_motion(self, x, y, dx, dy):
        if self.__new:
            m_x, m_y = director.get_virtual_coordinates(x,y)
            w_x, w_y = self.__to_world(m_x, m_y)
            c = rabbyt.collisions.collide_single((m_x, m_y), self.__co)
            for i in c:
                self.__new.xy = self.__to_world(i.x, i.y)
            return True
        return self.__buttonActions('on_mouse_motion', x, y, dx, dy)
    
    def on_mouse_release(self, x, y, button, modifiers):
        r = self.__buttonActions('on_mouse_release', x, y, button, modifiers)
        
        if not r:
            r = self.__delete_btn.on_mouse_release(x, y, button, modifiers)
        if not r:
            if button == mouse.LEFT:
                if self.__tracked_tile is not None:
                    self.__tracked_tile.alpha = 1
                    self.__tracked_tile = None
                elif modifiers & key.MOD_SHIFT:
                    for i in self.__selected:
                        x = rabbyt.collisions.collide_single(i, self.__co)
                        if x:
                            i.xy = x[-1].xy
            elif button == mouse.RIGHT:
                mx, my = self.__to_world(*director.get_virtual_coordinates(x,y))
                return self.__select_tile(mx,my)
            elif button == mouse.MIDDLE:
                mx, my = director.get_virtual_coordinates(x,y)
                if not self.__starting_pos:
                    self.__starting_pos = (0,0)
                mx = (round((mx - self.__starting_pos[0]) / 64.0) * 64) - 32
                my = (round((my - self.__starting_pos[1]) / 64.0) * 64) - 32
                
                self.__camera.setPosition(-mx, -my)
                self.__starting_pos = None
                return True
            return False
        return r
    
    def absxrange(self, x, y, step=1):
        print x, y
        i, j = min(x,y), max(x,y)
        print i,j
        return xrange(i, j, step)
    
    def __select_tile(self, x, y, allow_deselect=True):
        c = rabbyt.collisions.collide_single((x,y), self.__obj)
        if c:
            for i in self.__selected:
                if (i in c) and allow_deselect:
                    self.__selected.remove(i)
                    i.red = 1
                    return True
            i = c[-1]
            i.red = 0.1

            self.__selected.append(i)
            return True
        return False
    
    def on_mouse_drag(self, x, y, dx, dy, button, modifiers):
        m_x, m_y = director.get_virtual_coordinates(x, y)
        if mouse.LEFT == button:
            if modifiers & key.MOD_SHIFT:
                for i in self.__selected:
                    i.x += dx
                    i.y += dy
            elif self.__tracked_tile is not None:
                c = rabbyt.collisions.collide_single((m_x, m_y), self.__co)
                for i in c:
                    self.__tracked_tile.xy = self.__to_world(i.x, i.y)
                    return True
        elif mouse.MIDDLE == button:
            self.__camera.translate(-dx, -dy)
            return True
        elif (mouse.RIGHT == button) and modifiers & key.MOD_SHIFT:
            w_x, w_y = self.__to_world(x, y)
            return self.__select_tile(w_x, w_y, False)
        return False
    
    def on_key_release(self, symbol, modifiers):
        if (modifiers & key.MOD_CTRL) and (symbol == key.C):
            for i in self.__selected:
                i.red = 1
            self.__selected = []

def main():
    director.init(width=sWidth, 
                  height=sHeight, 
                  caption="ProjectX Editor")
    
    rabbyt.set_viewport((sWidth, sHeight))
    rabbyt.set_default_attribs()
    
    t = Terrain()
    
    MenuLayer = cocos.layer.MultiplexLayer(Main(t), NewTile(t))
    
    scene = cocos.scene.Scene(Grid(t), t, MenuLayer)

    director.run(scene)

if __name__ == "__main__":
    main()
